@@ -33,9 +33,13 @@ def _test_rgba16(import_local_tifffile=False):
     nt.assert_equal(s.axes_manager[2].units, t.Undefined)
     nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
     nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=5)
-    nt.assert_almost_equal(s.axes_manager[2].scale, 1.0, places=5)
+    nt.assert_almost_equal(s.axes_manager[2].scale, 1.0, places=5)        
 
 
+def _compare_signal_shape_data(s0, s1):
+    nt.assert_equal(s0.data.shape, s1.data.shape)
+    np.testing.assert_equal(s0.data, s1.data)
+
 def test_read_unit_um():
     # Load DM file and save it as tif
     s = hs.load(os.path.join(my_path2, 'test_dm_image_um_unit.dm3'))
@@ -77,6 +81,76 @@ def _test_read_unit_from_imagej(import_local_tifffile=False):
     nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
 
 
+def test_read_unit_from_imagej_stack(import_local_tifffile=False):
+    fname = os.path.join(my_path, 'tiff_files',
+                         'test_loading_image_saved_with_imageJ_stack.tif')
+    s = hs.load(fname, import_local_tifffile=import_local_tifffile)
+    nt.assert_equal(s.data.shape, (2, 68, 68))
+    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
+    nt.assert_equal(s.axes_manager[1].units, 'micron')
+    nt.assert_equal(s.axes_manager[2].units, 'micron')
+    nt.assert_almost_equal(s.axes_manager[0].scale, 2.5, places=5)
+    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
+    nt.assert_almost_equal(s.axes_manager[2].scale, 0.16867, places=5)
+
+
+def test_read_unit_from_DM_stack(import_local_tifffile=False):
+    fname = os.path.join(my_path, 'tiff_files',
+                         'test_loading_image_saved_with_DM_stack.tif')
+    s = hs.load(fname, import_local_tifffile=import_local_tifffile)
+    nt.assert_equal(s.data.shape, (2, 68, 68))
+    nt.assert_equal(s.axes_manager[0].units, 's')
+    nt.assert_equal(s.axes_manager[1].units, 'µm')
+    nt.assert_equal(s.axes_manager[2].units, 'µm')
+    nt.assert_almost_equal(s.axes_manager[0].scale, 2.5, places=5)
+    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
+    nt.assert_almost_equal(s.axes_manager[2].scale, 1.68674, places=5)
+    fname2 = fname.replace('DM_stack.tif', 'DM_stack2.tif')
+    s.save(fname2, overwrite=True)
+    s2 = hs.load(fname2)
+    _compare_signal_shape_data(s, s2)
+    nt.assert_equal(s2.axes_manager[0].units, s.axes_manager[0].units)
+    nt.assert_equal(s2.axes_manager[1].units, 'micron')
+    nt.assert_equal(s2.axes_manager[2].units, 'micron')
+    nt.assert_almost_equal(s2.axes_manager[0].scale, s.axes_manager[0].scale,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[1].scale, s.axes_manager[1].scale,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[2].scale, s.axes_manager[2].scale,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[0].offset, s.axes_manager[0].offset,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[1].offset, s.axes_manager[1].offset,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[2].offset, s.axes_manager[2].offset,
+                           places=5)
+    if remove_files:
+        os.remove(fname2)    
+
+    
+def test_read_unit_from_imagej_stack_no_scale(import_local_tifffile=False):
+    fname = os.path.join(my_path, 'tiff_files',
+        'test_loading_image_saved_with_imageJ_stack_no_scale.tif')
+    s = hs.load(fname, import_local_tifffile=import_local_tifffile)
+    nt.assert_equal(s.data.shape, (2, 68, 68))
+    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
+    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
+    nt.assert_equal(s.axes_manager[2].units, t.Undefined)
+    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
+    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=5)
+    nt.assert_almost_equal(s.axes_manager[2].scale, 1.0, places=5)
+        
+
+def test_read_unit_from_imagej_no_scale(import_local_tifffile=False):
+    fname = os.path.join(my_path, 'tiff_files',
+                         'test_loading_image_saved_with_imageJ_no_scale.tif')
+    s = hs.load(fname, import_local_tifffile=import_local_tifffile)
+    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
+    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
+    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
+    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=5)
+    
+
 def test_write_read_unit_imagej(import_local_tifffile=True):
     fname = os.path.join(my_path, 'tiff_files',
                          'test_loading_image_saved_with_imageJ.tif')
@@ -86,8 +160,9 @@ def test_write_read_unit_imagej(import_local_tifffile=True):
     fname2 = fname.replace('.tif', '2.tif')
     s.save(fname2, export_scale=True, overwrite=True)
     s2 = hs.load(fname2, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s2.axes_manager[0].units, 'µm')
-    nt.assert_equal(s2.axes_manager[1].units, 'µm')
+    nt.assert_equal(s2.axes_manager[0].units, 'micron')
+    nt.assert_equal(s2.axes_manager[1].units, 'micron')
+    nt.assert_equal(s.data.shape, s.data.shape)
     if remove_files:
         os.remove(fname2)
 
@@ -111,8 +186,8 @@ def test_write_read_unit_imagej_with_description(import_local_tifffile=True):
     fname3 = fname.replace('.tif', '_description2.tif')
     s.save(fname3, export_scale=True, overwrite=True, description='test')
     s3 = hs.load(fname3, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s3.axes_manager[0].units, 'µm')
-    nt.assert_equal(s3.axes_manager[1].units, 'µm')
+    nt.assert_equal(s3.axes_manager[0].units, 'micron')
+    nt.assert_equal(s3.axes_manager[1].units, 'micron')
     nt.assert_almost_equal(s3.axes_manager[0].scale, 0.16867, places=5)
     nt.assert_almost_equal(s3.axes_manager[1].scale, 0.16867, places=5)
 
@@ -155,8 +230,26 @@ def _test_read_unit_from_dm(import_local_tifffile=False):
     nt.assert_equal(s.axes_manager[1].units, 'µm')
     nt.assert_almost_equal(s.axes_manager[0].scale, 0.16867, places=5)
     nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
+    nt.assert_almost_equal(s.axes_manager[0].offset, 139.66264, places=5)
+    nt.assert_almost_equal(s.axes_manager[1].offset, 128.19276, places=5)
+    fname2 = fname.replace('DM.tif', 'DM2.tif')
+    s.save(fname2, overwrite=True)
+    s2 = hs.load(fname2)
+    _compare_signal_shape_data(s, s2)
+    nt.assert_equal(s2.axes_manager[0].units, 'micron')
+    nt.assert_equal(s2.axes_manager[1].units, 'micron')
+    nt.assert_almost_equal(s2.axes_manager[0].scale, s.axes_manager[0].scale,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[1].scale, s.axes_manager[1].scale,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[0].offset, s.axes_manager[0].offset,
+                           places=5)
+    nt.assert_almost_equal(s2.axes_manager[1].offset, s.axes_manager[1].offset,
+                           places=5)
+    if remove_files:
+        os.remove(fname2)    
 
-
+    
 def test_write_scale_unit():
     _test_write_scale_unit(export_scale=True)
 
@@ -239,11 +332,11 @@ def test_write_scale_with_undefined_scale():
             dtype=np.uint8).reshape(
             (10,
              15)))
-    s.axes_manager[0].name = 'x'
-    s.axes_manager[1].name = 'y'
     fname = os.path.join(my_path, 'tiff_files',
                          'test_export_scale_undefined_scale.tif')
     s.save(fname, overwrite=True, export_scale=True)
+    s1 = hs.load(fname)
+    _compare_signal_shape_data(s, s1)
     if remove_files:
         os.remove(fname)
 
@@ -255,6 +348,8 @@ def test_write_scale_with_um_unit():
                              'test_dm_image_um_unit.dm3'))
     fname = os.path.join(my_path, 'tiff_files', 'test_export_um_unit.tif')
     s.save(fname, overwrite=True, export_scale=True)
+    s1 = hs.load(fname)
+    _compare_signal_shape_data(s, s1)
     if remove_files:
         os.remove(fname)
 
@@ -269,21 +364,37 @@ def test_write_scale_unit_image_stack():
             (5,
              10,
              15)))
-    s.axes_manager[0].name = 'x'
-    s.axes_manager[1].name = 'y'
-    s.axes_manager[2].name = 'z'
-    s.axes_manager['x'].scale = 0.25
-    s.axes_manager['y'].scale = 0.5
-    s.axes_manager['z'].scale = 1.5
-    s.axes_manager['x'].units = 'nm'
-    s.axes_manager['y'].units = 'um'
-    s.axes_manager['z'].units = 'mm'
+    s.axes_manager[0].scale = 0.25
+    s.axes_manager[1].scale = 0.5
+    s.axes_manager[2].scale = 1.5
+    s.axes_manager[0].units = 'nm'
+    s.axes_manager[1].units = 'um'
+    s.axes_manager[2].units = 'mm'
     fname = os.path.join(my_path, 'tiff_files',
-                         'test_export_scale_unit_stack.tif')
+                         'test_export_scale_unit_stack2.tif')
     s.save(fname, overwrite=True, export_scale=True)
+    s1 = hs.load(fname)
+    _compare_signal_shape_data(s, s1)
+    nt.assert_equal(s1.axes_manager[0].units, 'nm')
+    nt.assert_equal(s1.axes_manager[1].units, 'mm') # only one unit can be read
+    nt.assert_equal(s1.axes_manager[2].units, 'mm')
+    nt.assert_almost_equal(s1.axes_manager[0].scale, s.axes_manager[0].scale)
+    nt.assert_almost_equal(s1.axes_manager[1].scale, s.axes_manager[1].scale)
+    nt.assert_almost_equal(s1.axes_manager[2].scale, s.axes_manager[2].scale)
     if remove_files:
         os.remove(fname)
 
+        
+def test_saving_loading_stack_no_scale():
+    fname = os.path.join(my_path, 'tiff_files',
+                         'test_export_scale_unit_stack2.tif')
+    s0 = hs.signals.Signal2D(np.zeros((10, 20, 30)))
+    s0.save(fname, overwrite=True)
+    s1 = hs.load(fname)
+    _compare_signal_shape_data(s0, s1)
+    if remove_files:
+        os.remove(fname)
+        
 
 def test_read_FEI_SEM_scale_metadata_8bits():
     fname = os.path.join(my_path2, 'FEI-Helios-Ebeam-8bits.tif')
@@ -348,7 +459,7 @@ def test_read_BW_Zeiss_optical_scale_metadata():
     nt.assert_almost_equal(s.axes_manager[1].scale, 169.3333, places=3)
 
 
-def test_read_BW_Zeiss_optical_scale_metadata_old():
+def test_read_BW_Zeiss_optical_scale_metadata2():
     fname = os.path.join(my_path2, 'optical_Zeiss_AxioVision_BW.tif')
     s = hs.load(fname, force_read_resolution=True)
     nt.assert_equal(s.data.dtype, np.uint16)
@@ -359,7 +470,7 @@ def test_read_BW_Zeiss_optical_scale_metadata_old():
     nt.assert_almost_equal(s.axes_manager[1].scale, 169.3333, places=3)
 
 
-def test_read_BW_Zeiss_optical_scale_metadata_old2():
+def test_read_BW_Zeiss_optical_scale_metadata3():
     fname = os.path.join(my_path2, 'optical_Zeiss_AxioVision_BW.tif')
     s = hs.load(fname, force_read_resolution=False)
     nt.assert_equal(s.data.dtype, np.uint16)
